// package ch.epfl.compnet;
import java.io.*;
import java.nio.file.*;
import java.net.*;
import java.util.*;

public class TCPClient {
    private static FileInputStream getFileReader(String filename) {
        FileInputStream fis = null;
        boolean fileExists = true;
        try {
            fis = new FileInputStream(filename);
        } catch (FileNotFoundException e) {
            fileExists = false;
        }
        return fis;
    }

    private static int getFileLength(String filename) {
        File file = new File(filename);
        int length = (int) file.length();
        return length;
    }

    private static void printHashMap(Map<String, Integer> occurrences) {
        for (String name: occurrences.keySet()) {
            String key = name.toString();
            String value = occurrences.get(name).toString();
            System.out.println(key + " " + value);
        }
    }

    private static void sendBytes(FileInputStream fis, OutputStream os) throws IOException {
        // Construct a 1K buffer to hold bytes on their way to the socket.
        byte[] buffer = new byte[1024];
        int bytes = 0;

        // Copy requested file into the socket's output stream.
        while ((bytes = fis.read(buffer)) != -1) {
            os.write(buffer, 0, bytes);
        }
    }

    private static boolean sendFile(DataOutputStream os, String filename) throws IOException {
        boolean retval;

        if (filename.isEmpty() || !Files.isReadable(Paths.get(filename))) {
            retval = false;
        } else {
            retval = true;

            // Send the file itself
            FileInputStream fis = getFileReader(filename);
            sendBytes(fis, os);
        }
        return retval;
    }

    public static void main(String argv[]) {
        Socket clientSocket = null;

        BufferedReader inFromUser = null;
        DataOutputStream outToServer = null;

        try {
            // Connect to the local server at 6789
            clientSocket = new Socket("localhost", 6789);

            inFromUser = new BufferedReader(new InputStreamReader(System.in));
            outToServer = new DataOutputStream(clientSocket.getOutputStream());

            System.out.println("Connected to server");
            System.out.print("Enter a file name: ");
            String filename = inFromUser.readLine();

            // sendfile will notify us whether this is the final file or not
            sendFile(outToServer, filename);
        } catch (IOException ioex) {
            System.out.println("Failed to process request : " + ioex.getMessage());
        }
    }
}